

#include <cv.h>
#include <highgui.h>
#include <cvaux.h>

#include "objectTracker.h"

IplImage* withRect = 0;
IplImage* vidImg = 0;
IplImage* plotCenter = 0;

CvRect inputRect;

int X1,Y1,X2,Y2;

bool selectedROI;

void drawRect(){
	
	withRect = cvCloneImage(vidImg);	
	cvRectangle(withRect, cvPoint(X1, Y1), cvPoint(X2, Y2), CV_RGB(0,0,255),1);
	cvShowImage( "vidImg", withRect );
	cvReleaseImage(&withRect);
}

void mouseHandler(int event, int x, int y, int flags, void* param)
{
	
	switch (event){
		case CV_EVENT_LBUTTONDOWN:
			X1 = X2 = x;
			Y1 = Y2 = y;
			break;

		case CV_EVENT_MOUSEMOVE:
			if(flags & CV_EVENT_FLAG_LBUTTON){	//dragging
				X2 = x;
				Y2 = y;
				if(X2 < 0) X2 = 0;
				if(X2 >= vidImg->width) X2 = vidImg->width - 1;
				if(Y2 < 0) Y2 = 0;
				if(Y2 >= vidImg->height) Y2 =vidImg->height - 1;
				drawRect();
			}
			break;

		case CV_EVENT_LBUTTONUP:
			inputRect.x = MIN(X1,X2);
			inputRect.y = MIN(Y1,Y2);
			inputRect.width = abs(X2-X1);
			inputRect.height = abs(Y2-Y1);
			if(inputRect.width == 0) inputRect.width = 1;
			if(inputRect.height == 0) inputRect.height = 1;
			selectedROI = true;
			break;
	}
	
}


int main(int argc, char** argv){
	
	int key = -1;
	
	ObjectTracker videoTracker; 

	CvCapture* capture = cvCreateFileCapture(argv[1]); 
	if( !capture){
		printf("Could not load file: %s\n", argv[1]); 
		exit(0);
	}

	vidImg = cvQueryFrame(capture);           // retrieve first frame
	cvNamedWindow( "vidImg", 1 );			//create window
	cvMoveWindow("vidImg", 100, 100);
	cvShowImage( "vidImg", vidImg );		//show first frame
	cvSetMouseCallback("vidImg", mouseHandler, NULL);

	//wait until user selects region to track and then push the space key
	printf( "File loaded...\n");
	printf( "Please select a region to track then press the space bar.\n");
	while(key != ' ' && !selectedROI) key = cvWaitKey();

	//initialize plot image and window
	plotCenter = cvCreateImage( cvGetSize(vidImg), IPL_DEPTH_8U, 3);
	cvZero(plotCenter);
	cvNamedWindow( "plotCenter", 1 );			//create window
	cvMoveWindow("plotCenter", cvGetCaptureProperty(capture, CV_CAP_PROP_FRAME_WIDTH) + 110, 100);
	cvShowImage( "plotCenter", plotCenter );		//show first frame

	videoTracker.init(vidImg, inputRect);	//this had to be moved below the wait...since inputRect wasn't set yet (user didn't trigger callback)

	int fps = ( int )cvGetCaptureProperty( capture, CV_CAP_PROP_FPS );
	//CvVideoWriter* writer = cvCreateVideoWriter("out.avi", -1, fps, cvGetSize(vidImg));
	CvPoint pcen;

	for(int i = 0; i < cvGetCaptureProperty(capture, CV_CAP_PROP_FRAME_COUNT); i++){
		
		vidImg = cvQueryFrame(capture);           // retrieve next frame
		if(!vidImg) break;

		CvBox2D box = videoTracker.track(vidImg);			//locate object in next image
		CvPoint cen = cvPoint(box.center.x,box.center.y);	//extract center location

		cvEllipseBox(vidImg, box, CV_RGB(0,255,0), 2, CV_AA, 0 );  //outline object with ellipse
		cvCircle(vidImg, cen, 1, CV_RGB(0,255,0),2);				//draw centroid
		cvShowImage( "vidImg", vidImg );
		//cvWriteFrame(writer, vidImg);		//write to output video

		if( i!=0) cvLine(plotCenter, pcen, cen, CV_RGB(0,255,0));	//plot center connected each frame
		cvShowImage( "plotCenter", plotCenter);
		printf("x=%d, y=%d\n", cen.x, cen.y);	//print center coords
		pcen = cen;

		key = cvWaitKey(1000/fps); //wait to maintain framerate
		if( key == 'q') break;		//stop video if q is pressed
	}	

	cvReleaseImage( &vidImg);
	cvReleaseCapture( &capture);	
	//cvReleaseVideoWriter( &writer);

	printf("press any key...\n");
	cvWaitKey();
	
}
