/*
 *  objectTracker.cpp
 *  opencv object tracking
 *
 *  Created by Ryan McGee and Tim Wood on 3/7/09.
 *
 */

#include <cv.h>
#include <highgui.h>
#include <cvaux.h>

class ObjectTracker {
	public:
		
	ObjectTracker(); //constructor
	~ObjectTracker(); //destructor
	
	void init(IplImage* image, CvRect inputRect); // initialize object tracker
	
	CvBox2D track(IplImage* image); //tracking function, returns tracking box
	
	
	private:
		int   nHistBins;           // number of histogram bins
		float histRange[2];         // histogram range
		int valMin, valMax, satMin;     // hue limits
	
		// File-level variables
		IplImage * hsvImg; // the input image converted to HSV color mode
		IplImage * hueImg; // the Hue channel of the HSV image
		IplImage * mask; // this image is used for masking pixels
		IplImage * probImg; // the object probability estimates for each pixel
		CvHistogram * hist; // histogram of hue in the original image 
	
		CvRect prevTrackRect;  // location of object in previous frame
		CvBox2D trackBox;      // current object location estimate
	
		void convertToHSV(IplImage* image);
	
};